# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.org_structure_to_update_container import OrgStructureToUpdateContainer
from ..models.org_structure_to_create_container import OrgStructureToCreateContainer
from ..models.get_org_structure_container import GetOrgStructureContainer
from ..models.get_all_org_structures_container import GetAllOrgStructuresContainer


class OrganizationStructureService(BaseService):

    @cast_models
    def get_all_org_structures(self) -> GetAllOrgStructuresContainer:
        """Fetch all Org Structures from the Organization.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GetAllOrgStructuresContainer
        """

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/organization_structures",
                self.get_default_headers(),
            )
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GetAllOrgStructuresContainer._unmap(response)

    @cast_models
    def create_org_structure(
        self, request_body: OrgStructureToCreateContainer
    ) -> GetAllOrgStructuresContainer:
        """Create a new HRIS Organization Structure.

        :param request_body: The request body.
        :type request_body: OrgStructureToCreateContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GetAllOrgStructuresContainer
        """

        Validator(OrgStructureToCreateContainer).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/organization_structures",
                self.get_default_headers(),
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return GetAllOrgStructuresContainer._unmap(response)

    @cast_models
    def get_org_structure(self, hris_org_str_id: str) -> GetOrgStructureContainer:
        """Fetch an Org Structure from the Organization.

        :param hris_org_str_id: HRIS Organization Structure ID.
        :type hris_org_str_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GetOrgStructureContainer
        """

        Validator(str).validate(hris_org_str_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/organization_structures/{{hrisOrgStr_id}}",
                self.get_default_headers(),
            )
            .add_path("hrisOrgStr_id", hris_org_str_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GetOrgStructureContainer._unmap(response)

    @cast_models
    def update_org_structure(
        self, request_body: OrgStructureToUpdateContainer, hris_org_str_id: str
    ) -> any:
        """Update an existing Organization structure.

        :param request_body: The request body.
        :type request_body: OrgStructureToUpdateContainer
        :param hris_org_str_id: HRIS Organization Structure ID.
        :type hris_org_str_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: any
        """

        Validator(OrgStructureToUpdateContainer).validate(request_body)
        Validator(str).validate(hris_org_str_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/organization_structures/{{hrisOrgStr_id}}",
                self.get_default_headers(),
            )
            .add_path("hrisOrgStr_id", hris_org_str_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def delete_org_structure(self, hris_org_str_id: str) -> any:
        """Delete an Org Structure from the Organization.

        :param hris_org_str_id: HRIS Organization Structure ID.
        :type hris_org_str_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successfully deleted.
        :rtype: any
        """

        Validator(str).validate(hris_org_str_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/organization_structures/{{hrisOrgStr_id}}",
                self.get_default_headers(),
            )
            .add_path("hrisOrgStr_id", hris_org_str_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def get_external_org_structure(self, external_id: str) -> GetOrgStructureContainer:
        """Fetch an Org Structure from the Organization.

        :param external_id: External ID.
        :type external_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GetOrgStructureContainer
        """

        Validator(str).validate(external_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/organization_structures/external/{{external_id}}",
                self.get_default_headers(),
            )
            .add_path("external_id", external_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GetOrgStructureContainer._unmap(response)

    @cast_models
    def update_external_org_structure(
        self, request_body: OrgStructureToUpdateContainer, external_id: str
    ) -> any:
        """Update an existing Organization structure.

        :param request_body: The request body.
        :type request_body: OrgStructureToUpdateContainer
        :param external_id: External ID.
        :type external_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: any
        """

        Validator(OrgStructureToUpdateContainer).validate(request_body)
        Validator(str).validate(external_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/organization_structures/external/{{external_id}}",
                self.get_default_headers(),
            )
            .add_path("external_id", external_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def delete_external_org_structure(self, external_id: str) -> any:
        """Delete an Org Structure from the Organization.

        :param external_id: External ID.
        :type external_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successfully deleted.
        :rtype: any
        """

        Validator(str).validate(external_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/organization_structures/external/{{external_id}}",
                self.get_default_headers(),
            )
            .add_path("external_id", external_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return response



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from typing import Union
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel


@JsonMap({})
class Data12(BaseModel):
    """Text type

    :param value: value, defaults to None
    :type value: str, optional
    """

    def __init__(self, value: str = None):
        if value is not None:
            self.value = value


@JsonMap({})
class Data22(BaseModel):
    """Number type

    :param value: value, defaults to None
    :type value: float, optional
    """

    def __init__(self, value: float = None):
        if value is not None:
            self.value = value


@JsonMap({})
class Data32(BaseModel):
    """Date type: YYYY-MM-DD

    :param value: value, defaults to None
    :type value: str, optional
    """

    def __init__(self, value: str = None):
        if value is not None:
            self.value = value


@JsonMap({})
class Data42(BaseModel):
    """Percentage type

    :param value: value, defaults to None
    :type value: float, optional
    """

    def __init__(self, value: float = None):
        if value is not None:
            self.value = value


class Data5Currency2(Enum):
    """An enumeration representing different categories.

    :cvar USD: "USD"
    :vartype USD: str
    :cvar EUR: "EUR"
    :vartype EUR: str
    :cvar JPY: "JPY"
    :vartype JPY: str
    :cvar GBP: "GBP"
    :vartype GBP: str
    :cvar AUD: "AUD"
    :vartype AUD: str
    :cvar CAD: "CAD"
    :vartype CAD: str
    :cvar CHF: "CHF"
    :vartype CHF: str
    :cvar CNY: "CNY"
    :vartype CNY: str
    :cvar SEK: "SEK"
    :vartype SEK: str
    :cvar NZD: "NZD"
    :vartype NZD: str
    :cvar BRL: "BRL"
    :vartype BRL: str
    """

    USD = "USD"
    EUR = "EUR"
    JPY = "JPY"
    GBP = "GBP"
    AUD = "AUD"
    CAD = "CAD"
    CHF = "CHF"
    CNY = "CNY"
    SEK = "SEK"
    NZD = "NZD"
    BRL = "BRL"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Data5Currency2._member_map_.values()))


@JsonMap({})
class Data52(BaseModel):
    """Currency type

    :param amount: amount, defaults to None
    :type amount: float, optional
    :param currency: Important: When currency type is fixed, the currency field is ignored. , defaults to None
    :type currency: Data5Currency2, optional
    """

    def __init__(self, amount: float = None, currency: Data5Currency2 = None):
        if amount is not None:
            self.amount = amount
        if currency is not None:
            self.currency = self._enum_matching(
                currency, Data5Currency2.list(), "currency"
            )


@JsonMap({})
class Data62(BaseModel):
    """List type

    :param option: option, defaults to None
    :type option: str, optional
    """

    def __init__(self, option: str = None):
        if option is not None:
            self.option = option


@JsonMap({})
class Data72(BaseModel):
    """Multiselect type

    :param options: options, defaults to None
    :type options: List[str], optional
    """

    def __init__(self, options: List[str] = None):
        if options is not None:
            self.options = options


class CustomFieldToUpdateContainerDataGuard(OneOfBaseModel):
    class_list = {
        "Data12": Data12,
        "Data22": Data22,
        "Data32": Data32,
        "Data42": Data42,
        "Data52": Data52,
        "Data62": Data62,
        "Data72": Data72,
    }


CustomFieldToUpdateContainerData = Union[
    Data12, Data22, Data32, Data42, Data52, Data62, Data72
]


@JsonMap({"id_": "id"})
class CustomFieldToUpdateContainer(BaseModel):
    """CustomFieldToUpdateContainer

    :param id_: UUID for the custom field value.
    :type id_: str
    :param data: data
    :type data: CustomFieldToUpdateContainerData
    """

    def __init__(self, id_: str, data: CustomFieldToUpdateContainerData):
        self.id_ = id_
        self.data = CustomFieldToUpdateContainerDataGuard.return_one_of(data)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id", "hris_org_structure_id": "hrisOrgStructureId"})
class GetRolesItem(BaseModel):
    """GetRolesItem

    :param id_: id_, defaults to None
    :type id_: float, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param scope: scope, defaults to None
    :type scope: str, optional
    :param hris_org_structure_id: hris_org_structure_id, defaults to None
    :type hris_org_structure_id: str, optional
    """

    def __init__(
        self,
        id_: float = None,
        name: str = None,
        scope: str = None,
        hris_org_structure_id: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if scope is not None:
            self.scope = scope
        if hris_org_structure_id is not None:
            self.hris_org_structure_id = hris_org_structure_id



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class ContractTerminationResult(BaseModel):
    """ContractTerminationResult

    :param id_: Deel Contract ID., defaults to None
    :type id_: str, optional
    :param completion_date: Long date-time format following ISO-8601, defaults to None
    :type completion_date: str, optional
    """

    def __init__(self, id_: str = None, completion_date: str = None):
        if id_ is not None:
            self.id_ = id_
        if completion_date is not None:
            self.completion_date = completion_date



# This file was generated by liblab | https://liblab.com/

from typing import Any, Dict
from enum import Enum

from .default_headers import DefaultHeaders, DefaultHeadersKeys
from ...net.transport.request import Request
from ...net.request_chain.request_chain import RequestChain
from ...net.request_chain.handlers.hook_handler import HookHandler
from ...net.request_chain.handlers.http_handler import HttpHandler
from ...net.headers.access_token_auth import AccessTokenAuth
from ...net.request_chain.handlers.retry_handler import RetryHandler


class BaseService:
    """
    A base class for services providing common functionality.

    :ivar str base_url: The base URL for the service.
    :ivar dict _default_headers: A dictionary of default headers.
    """

    def __init__(self, base_url: str) -> None:
        """
        Initializes a BaseService instance.

        :param str base_url: The base URL for the service. Defaults to None.
        """
        self.base_url = base_url
        self._default_headers = DefaultHeaders()

        self._request_handler = self._get_request_handler()

    def set_access_token(self, access_token: str):
        """
        Sets the access token for the service.
        """
        self._default_headers.set_header(
            DefaultHeadersKeys.ACCESS_AUTH, AccessTokenAuth(access_token)
        )

        return self

    def set_base_url(self, base_url: str):
        """
        Sets the base URL for the service.

        :param str base_url: The base URL to be set.
        """
        self.base_url = base_url

        return self

    def send_request(self, request: Request) -> dict:
        """
        Sends the given request.

        :param Request request: The request to be sent.
        :return: The response data.
        :rtype: dict
        """
        response = self._request_handler.send(request)
        return response.body

    def get_default_headers(self) -> list:
        """
        Get the default headers.

        :return: A list of the default headers.
        :rtype: list
        """
        return self._default_headers.get_headers()

    def _get_request_handler(self) -> RequestChain:
        """
        Get the request chain.

        :return: The request chain.
        :rtype: RequestChain
        """
        return (
            RequestChain()
            .add_handler(RetryHandler())
            .add_handler(HookHandler())
            .add_handler(HttpHandler())
        )



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class Role(BaseModel):
    """Role

    :param id_: id_, defaults to None
    :type id_: int, optional
    :param name: name, defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: int = None, name: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name


@JsonMap({"id_": "id"})
class OrganizationStructure(BaseModel):
    """OrganizationStructure

    :param id_: id_, defaults to None
    :type id_: int, optional
    :param name: name, defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: int = None, name: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name


@JsonMap({"id_": "id"})
class DataTeam(BaseModel):
    """DataTeam

    :param id_: id_, defaults to None
    :type id_: int, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param organization_structure: organization_structure, defaults to None
    :type organization_structure: OrganizationStructure, optional
    """

    def __init__(
        self,
        id_: int = None,
        name: str = None,
        organization_structure: OrganizationStructure = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if organization_structure is not None:
            self.organization_structure = self._define_object(
                organization_structure, OrganizationStructure
            )


@JsonMap({"id_": "id"})
class GetHrisPositionContainerData(BaseModel):
    """GetHrisPositionContainerData

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param role: role, defaults to None
    :type role: Role, optional
    :param team: team, defaults to None
    :type team: DataTeam, optional
    :param is_supportive: is_supportive, defaults to None
    :type is_supportive: bool, optional
    """

    def __init__(
        self,
        id_: str = None,
        role: Role = None,
        team: DataTeam = None,
        is_supportive: bool = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if role is not None:
            self.role = self._define_object(role, Role)
        if team is not None:
            self.team = self._define_object(team, DataTeam)
        if is_supportive is not None:
            self.is_supportive = is_supportive


@JsonMap({})
class GetHrisPositionContainer(BaseModel):
    """GetHrisPositionContainer

    :param data: data, defaults to None
    :type data: List[GetHrisPositionContainerData], optional
    """

    def __init__(self, data: List[GetHrisPositionContainerData] = None):
        if data is not None:
            self.data = self._define_list(data, GetHrisPositionContainerData)



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class DataRoles1(BaseModel):
    """DataRoles1

    :param id_: id_, defaults to None
    :type id_: int, optional
    :param name: name, defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: int = None, name: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name


@JsonMap({"id_": "id", "type_": "type"})
class GetAllOrgStructuresContainerData(BaseModel):
    """GetAllOrgStructuresContainerData

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param type_: type_, defaults to None
    :type type_: str, optional
    :param roles: roles, defaults to None
    :type roles: List[DataRoles1], optional
    :param teams: teams, defaults to None
    :type teams: List[dict], optional
    :param external_id: external_id, defaults to None
    :type external_id: str, optional
    :param teams_count: teams_count, defaults to None
    :type teams_count: int, optional
    :param enable_roles: enable_roles, defaults to None
    :type enable_roles: bool, optional
    :param is_multiselect: is_multiselect, defaults to None
    :type is_multiselect: bool, optional
    """

    def __init__(
        self,
        id_: str = None,
        name: str = None,
        type_: str = None,
        roles: List[DataRoles1] = None,
        teams: List[dict] = None,
        external_id: str = None,
        teams_count: int = None,
        enable_roles: bool = None,
        is_multiselect: bool = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if type_ is not None:
            self.type_ = type_
        if roles is not None:
            self.roles = self._define_list(roles, DataRoles1)
        if teams is not None:
            self.teams = teams
        if external_id is not None:
            self.external_id = external_id
        if teams_count is not None:
            self.teams_count = teams_count
        if enable_roles is not None:
            self.enable_roles = enable_roles
        if is_multiselect is not None:
            self.is_multiselect = is_multiselect


@JsonMap({})
class GetAllOrgStructuresContainer(BaseModel):
    """GetAllOrgStructuresContainer

    :param data: data, defaults to None
    :type data: List[GetAllOrgStructuresContainerData], optional
    """

    def __init__(self, data: List[GetAllOrgStructuresContainerData] = None):
        if data is not None:
            self.data = self._define_list(data, GetAllOrgStructuresContainerData)



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap(
    {"permission_sets": "permissionSets", "hris_org_structure_id": "hrisOrgStructureId"}
)
class RoleToCreateContainerData(BaseModel):
    """RoleToCreateContainerData

    :param name: name
    :type name: str
    :param scope: scope
    :type scope: str
    :param description: description
    :type description: str
    :param permission_sets: permission_sets
    :type permission_sets: List[str]
    :param hris_org_structure_id: hris_org_structure_id, defaults to None
    :type hris_org_structure_id: str, optional
    """

    def __init__(
        self,
        name: str,
        scope: str,
        description: str,
        permission_sets: List[str],
        hris_org_structure_id: str = None,
    ):
        self.name = name
        self.scope = scope
        self.description = description
        self.permission_sets = permission_sets
        if hris_org_structure_id is not None:
            self.hris_org_structure_id = hris_org_structure_id


@JsonMap({})
class RoleToCreateContainer(BaseModel):
    """RoleToCreateContainer

    :param data: data, defaults to None
    :type data: RoleToCreateContainerData, optional
    """

    def __init__(self, data: RoleToCreateContainerData = None):
        if data is not None:
            self.data = self._define_object(data, RoleToCreateContainerData)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class ExerciseEquityToCreateContainer(BaseModel):
    """ExerciseEquityToCreateContainer

    :param fmv: fmv
    :type fmv: str
    :param quantity: quantity
    :type quantity: float
    :param issuer_id: issuer_id
    :type issuer_id: str
    :param worker_id: worker_id
    :type worker_id: str
    :param address_zip: address_zip
    :type address_zip: str
    :param exercise_id: exercise_id
    :type exercise_id: str
    :param address_city: address_city
    :type address_city: str
    :param address_state: address_state
    :type address_state: str
    :param address_street: address_street
    :type address_street: str
    :param stakeholder_id: stakeholder_id
    :type stakeholder_id: str
    :param address_country: address_country
    :type address_country: str
    :param option_grant_id: option_grant_id
    :type option_grant_id: str
    :param fmv_currency_code: fmv_currency_code
    :type fmv_currency_code: str
    :param stakeholder_email: stakeholder_email
    :type stakeholder_email: str
    :param option_grant_issue: option_grant_issue
    :type option_grant_issue: str
    :param option_grant_label: option_grant_label
    :type option_grant_label: str
    :param option_grant_currency: option_grant_currency
    :type option_grant_currency: str
    :param option_grant_issue_date: option_grant_issue_date
    :type option_grant_issue_date: str
    :param option_grant_expiry_date: option_grant_expiry_date
    :type option_grant_expiry_date: str
    :param stakeholder_relationship: stakeholder_relationship
    :type stakeholder_relationship: str
    :param option_grant_strike_price: option_grant_strike_price
    :type option_grant_strike_price: str
    """

    def __init__(
        self,
        fmv: str,
        quantity: float,
        issuer_id: str,
        worker_id: str,
        address_zip: str,
        exercise_id: str,
        address_city: str,
        address_state: str,
        address_street: str,
        stakeholder_id: str,
        address_country: str,
        option_grant_id: str,
        fmv_currency_code: str,
        stakeholder_email: str,
        option_grant_issue: str,
        option_grant_label: str,
        option_grant_currency: str,
        option_grant_issue_date: str,
        option_grant_expiry_date: str,
        stakeholder_relationship: str,
        option_grant_strike_price: str,
    ):
        self.fmv = fmv
        self.quantity = quantity
        self.issuer_id = issuer_id
        self.worker_id = worker_id
        self.address_zip = address_zip
        self.exercise_id = exercise_id
        self.address_city = address_city
        self.address_state = address_state
        self.address_street = address_street
        self.stakeholder_id = stakeholder_id
        self.address_country = address_country
        self.option_grant_id = option_grant_id
        self.fmv_currency_code = fmv_currency_code
        self.stakeholder_email = stakeholder_email
        self.option_grant_issue = option_grant_issue
        self.option_grant_label = option_grant_label
        self.option_grant_currency = option_grant_currency
        self.option_grant_issue_date = option_grant_issue_date
        self.option_grant_expiry_date = option_grant_expiry_date
        self.stakeholder_relationship = stakeholder_relationship
        self.option_grant_strike_price = option_grant_strike_price



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard
from .timesheet_status_enum import TimesheetStatusEnum
from .file_attachment_info import FileAttachmentInfo


@JsonMap({"id_": "id"})
class PgoTask(BaseModel):
    """PgoTask

    :param id_: id_
    :type id_: IdentifierValue
    :param amount: Fixed rate for this task.
    :type amount: str
    :param date_submitted: Short date in format ISO-8601 (YYYY-MM-DD). For example: 2022-12-31.
    :type date_submitted: str
    :param description: Task description.
    :type description: str
    :param status: Status of this entry.
    :type status: TimesheetStatusEnum
    :param attachment: This object is used for linking file attachments to your records., defaults to None
    :type attachment: FileAttachmentInfo, optional
    """

    def __init__(
        self,
        id_: IdentifierValue,
        amount: str,
        date_submitted: str,
        description: str,
        status: TimesheetStatusEnum,
        attachment: FileAttachmentInfo = None,
    ):
        self.id_ = IdentifierValueGuard.return_one_of(id_)
        self.amount = amount
        self.date_submitted = date_submitted
        self.description = description
        self.status = self._enum_matching(status, TimesheetStatusEnum.list(), "status")
        if attachment is not None:
            self.attachment = self._define_object(attachment, FileAttachmentInfo)



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.custom_fields_list_container import CustomFieldsListContainer
from ..models.custom_field_values_container import CustomFieldValuesContainer
from ..models.custom_field_to_update_container import CustomFieldToUpdateContainer
from ..models.custom_field_item_container import CustomFieldItemContainer


class CustomFieldsService(BaseService):

    @cast_models
    def get_custom_fields_from_people(self) -> CustomFieldsListContainer:
        """Fetch all custom fields associated with People records, offering additional personalized information.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful response.
        :rtype: CustomFieldsListContainer
        """

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/custom_fields", self.get_default_headers()
            )
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return CustomFieldsListContainer._unmap(response)

    @cast_models
    def get_custom_field_from_people_by_id(
        self, field_id: str
    ) -> CustomFieldItemContainer:
        """Access details of a specific custom field by using the custom field ID to enrich individual personnel records.

        :param field_id: Custom field id.
        :type field_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful response.
        :rtype: CustomFieldItemContainer
        """

        Validator(str).validate(field_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/custom_fields/{{field_id}}",
                self.get_default_headers(),
            )
            .add_path("field_id", field_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return CustomFieldItemContainer._unmap(response)

    @cast_models
    def get_custom_field_values_from_worker(
        self, worker_id: str
    ) -> CustomFieldValuesContainer:
        """Obtain all custom field values associated with a specific worker, facilitating detailed data retrieval for administrative or operational purposes.

        :param worker_id: Worker id.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful response.
        :rtype: CustomFieldValuesContainer
        """

        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/custom_fields",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return CustomFieldValuesContainer._unmap(response)

    @cast_models
    def upsert_custom_field_value_from_worker(
        self, request_body: CustomFieldToUpdateContainer, worker_id: str
    ):
        """Add or update custom field values for a specific worker, ensuring their profile is up-to-date with the latest information.

        :param request_body: The request body.
        :type request_body: CustomFieldToUpdateContainer
        :param worker_id: Worker id.
        :type worker_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(CustomFieldToUpdateContainer).validate(request_body)
        Validator(str).validate(worker_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/custom_fields",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def delete_custom_field_value_from_worker(self, worker_id: str, field_id: str):
        """Delete a specific custom field value from a worker's profile, removing outdated or unnecessary information.

        :param worker_id: Worker id.
        :type worker_id: str
        :param field_id: Custom field id.
        :type field_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(worker_id)
        Validator(str).validate(field_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/people/{{worker_id}}/custom_fields/{{field_id}}",
                self.get_default_headers(),
            )
            .add_path("worker_id", worker_id)
            .add_path("field_id", field_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def get_custom_fields_from_contracts(self) -> CustomFieldsListContainer:
        """Fetch all custom fields associated with contracts, providing additional data necessary for contract management.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful response.
        :rtype: CustomFieldsListContainer
        """

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/custom_fields", self.get_default_headers()
            )
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return CustomFieldsListContainer._unmap(response)

    @cast_models
    def get_custom_field_from_contracts_by_id(
        self, field_id: str
    ) -> CustomFieldItemContainer:
        """Access details of a specific custom field associated with a contract to manage contract-specific attributes effectively.

        :param field_id: Custom field id.
        :type field_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful response.
        :rtype: CustomFieldItemContainer
        """

        Validator(str).validate(field_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/custom_fields/{{field_id}}",
                self.get_default_headers(),
            )
            .add_path("field_id", field_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return CustomFieldItemContainer._unmap(response)

    @cast_models
    def get_custom_field_values_from_contract(
        self, contract_id: str
    ) -> CustomFieldValuesContainer:
        """Obtain all custom field values for a specific contract, enhancing contract management with detailed, custom-configured data.

        :param contract_id: Contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful response.
        :rtype: CustomFieldValuesContainer
        """

        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/custom_fields",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return CustomFieldValuesContainer._unmap(response)

    @cast_models
    def upsert_custom_field_value_from_contracts(
        self, request_body: CustomFieldToUpdateContainer, contract_id: str
    ):
        """Add or update custom field values for a specific contract, ensuring all contract details are current and relevant.

        :param request_body: The request body.
        :type request_body: CustomFieldToUpdateContainer
        :param contract_id: Contract id.
        :type contract_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(CustomFieldToUpdateContainer).validate(request_body)
        Validator(str).validate(contract_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/custom_fields",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .serialize()
            .set_method("PUT")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return response

    @cast_models
    def delete_contract_custom_field(self, contract_id: str, field_id: str):
        """Delete a specific custom field’s value from a contract, streamlining contract data management by removing redundant or outdated information.

        :param contract_id: Contract id.
        :type contract_id: str
        :param field_id: Custom field id.
        :type field_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(str).validate(contract_id)
        Validator(str).validate(field_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/contracts/{{contract_id}}/custom_fields/{{field_id}}",
                self.get_default_headers(),
            )
            .add_path("contract_id", contract_id)
            .add_path("field_id", field_id)
            .serialize()
            .set_method("DELETE")
        )

        response = self.send_request(serialized_request)

        return response



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class OrgStructureToUpdateContainerData(BaseModel):
    """OrgStructureToUpdateContainerData

    :param name: name, defaults to None
    :type name: str, optional
    :param teams: teams
    :type teams: List[dict]
    :param external_id: external_id, defaults to None
    :type external_id: str, optional
    :param enable_roles: enable_roles, defaults to None
    :type enable_roles: bool, optional
    :param is_multiselect: is_multiselect, defaults to None
    :type is_multiselect: bool, optional
    """

    def __init__(
        self,
        teams: List[dict],
        name: str = None,
        external_id: str = None,
        enable_roles: bool = None,
        is_multiselect: bool = None,
    ):
        if name is not None:
            self.name = name
        self.teams = teams
        if external_id is not None:
            self.external_id = external_id
        if enable_roles is not None:
            self.enable_roles = enable_roles
        if is_multiselect is not None:
            self.is_multiselect = is_multiselect


@JsonMap({})
class OrgStructureToUpdateContainer(BaseModel):
    """OrgStructureToUpdateContainer

    :param data: data, defaults to None
    :type data: OrgStructureToUpdateContainerData, optional
    """

    def __init__(self, data: OrgStructureToUpdateContainerData = None):
        if data is not None:
            self.data = self._define_object(data, OrgStructureToUpdateContainerData)



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from typing import Union
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel


@JsonMap({})
class Data11(BaseModel):
    """Text type

    :param value: value, defaults to None
    :type value: str, optional
    """

    def __init__(self, value: str = None):
        if value is not None:
            self.value = value


@JsonMap({})
class Data21(BaseModel):
    """Number type

    :param value: value, defaults to None
    :type value: float, optional
    """

    def __init__(self, value: float = None):
        if value is not None:
            self.value = value


@JsonMap({})
class Data31(BaseModel):
    """Date type: YYYY-MM-DD

    :param value: value, defaults to None
    :type value: str, optional
    """

    def __init__(self, value: str = None):
        if value is not None:
            self.value = value


@JsonMap({})
class Data41(BaseModel):
    """Percentage type

    :param text: text, defaults to None
    :type text: str, optional
    :param value: value, defaults to None
    :type value: float, optional
    """

    def __init__(self, text: str = None, value: float = None):
        if text is not None:
            self.text = text
        if value is not None:
            self.value = value


class Data5Currency1(Enum):
    """An enumeration representing different categories.

    :cvar USD: "USD"
    :vartype USD: str
    :cvar EUR: "EUR"
    :vartype EUR: str
    :cvar JPY: "JPY"
    :vartype JPY: str
    :cvar GBP: "GBP"
    :vartype GBP: str
    :cvar AUD: "AUD"
    :vartype AUD: str
    :cvar CAD: "CAD"
    :vartype CAD: str
    :cvar CHF: "CHF"
    :vartype CHF: str
    :cvar CNY: "CNY"
    :vartype CNY: str
    :cvar SEK: "SEK"
    :vartype SEK: str
    :cvar NZD: "NZD"
    :vartype NZD: str
    :cvar BRL: "BRL"
    :vartype BRL: str
    """

    USD = "USD"
    EUR = "EUR"
    JPY = "JPY"
    GBP = "GBP"
    AUD = "AUD"
    CAD = "CAD"
    CHF = "CHF"
    CNY = "CNY"
    SEK = "SEK"
    NZD = "NZD"
    BRL = "BRL"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Data5Currency1._member_map_.values()))


@JsonMap({})
class Data51(BaseModel):
    """Currency type

    :param amount: amount, defaults to None
    :type amount: float, optional
    :param currency: The currency code in ISO 4217 format, defaults to None
    :type currency: Data5Currency1, optional
    """

    def __init__(self, amount: float = None, currency: Data5Currency1 = None):
        if amount is not None:
            self.amount = amount
        if currency is not None:
            self.currency = self._enum_matching(
                currency, Data5Currency1.list(), "currency"
            )


@JsonMap({})
class Data61(BaseModel):
    """List type

    :param option: option, defaults to None
    :type option: str, optional
    """

    def __init__(self, option: str = None):
        if option is not None:
            self.option = option


@JsonMap({})
class Data71(BaseModel):
    """Multiselect type

    :param options: options, defaults to None
    :type options: List[str], optional
    """

    def __init__(self, options: List[str] = None):
        if options is not None:
            self.options = options


class DataDataGuard(OneOfBaseModel):
    class_list = {
        "Data11": Data11,
        "Data21": Data21,
        "Data31": Data31,
        "Data41": Data41,
        "Data51": Data51,
        "Data61": Data61,
        "Data71": Data71,
    }


DataData = Union[Data11, Data21, Data31, Data41, Data51, Data61, Data71]


class DataType(Enum):
    """An enumeration representing different categories.

    :cvar TEXT: "text"
    :vartype TEXT: str
    :cvar NUMBER: "number"
    :vartype NUMBER: str
    :cvar CURRENCY: "currency"
    :vartype CURRENCY: str
    :cvar DATE: "date"
    :vartype DATE: str
    :cvar PERCENTAGE: "percentage"
    :vartype PERCENTAGE: str
    :cvar LIST: "list"
    :vartype LIST: str
    :cvar MULTISELECT: "multiselect"
    :vartype MULTISELECT: str
    """

    TEXT = "text"
    NUMBER = "number"
    CURRENCY = "currency"
    DATE = "date"
    PERCENTAGE = "percentage"
    LIST = "list"
    MULTISELECT = "multiselect"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, DataType._member_map_.values()))


@JsonMap({"id_": "id", "type_": "type"})
class CustomFieldValuesContainerData(BaseModel):
    """CustomFieldValuesContainerData

    :param id_: UUID for the custom field value
    :type id_: str
    :param data: data, defaults to None
    :type data: DataData, optional
    :param name: Name of the custom field
    :type name: str
    :param type_: Type of the custom field
    :type type_: DataType
    """

    def __init__(self, id_: str, name: str, type_: DataType, data: DataData = None):
        self.id_ = id_
        if data is not None:
            self.data = DataDataGuard.return_one_of(data)
        self.name = name
        self.type_ = self._enum_matching(type_, DataType.list(), "type_")


@JsonMap({})
class CustomFieldValuesContainer(BaseModel):
    """CustomFieldValuesContainer

    :param data: data
    :type data: List[CustomFieldValuesContainerData]
    """

    def __init__(self, data: List[CustomFieldValuesContainerData]):
        self.data = self._define_list(data, CustomFieldValuesContainerData)



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id", "permission_sets": "permissionSets"})
class UpdatedRoleContainer(BaseModel):
    """UpdatedRoleContainer

    :param id_: id_, defaults to None
    :type id_: float, optional
    :param name: name, defaults to None
    :type name: str, optional
    :param scope: scope, defaults to None
    :type scope: str, optional
    :param description: description, defaults to None
    :type description: str, optional
    :param permission_sets: permission_sets, defaults to None
    :type permission_sets: List[str], optional
    """

    def __init__(
        self,
        id_: float = None,
        name: str = None,
        scope: str = None,
        description: str = None,
        permission_sets: List[str] = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if scope is not None:
            self.scope = scope
        if description is not None:
            self.description = description
        if permission_sets is not None:
            self.permission_sets = permission_sets



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .custom_fields_object import CustomFieldsObject


@JsonMap({})
class CustomFieldItemContainer(BaseModel):
    """CustomFieldItemContainer

    :param data: data
    :type data: CustomFieldsObject
    """

    def __init__(self, data: CustomFieldsObject):
        self.data = self._define_object(data, CustomFieldsObject)



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from typing import Union
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel


class Type1Name(Enum):
    """An enumeration representing different categories.

    :cvar TEXT: "text"
    :vartype TEXT: str
    :cvar NUMBER: "number"
    :vartype NUMBER: str
    :cvar DATE: "date"
    :vartype DATE: str
    :cvar PERCENTAGE: "percentage"
    :vartype PERCENTAGE: str
    """

    TEXT = "text"
    NUMBER = "number"
    DATE = "date"
    PERCENTAGE = "percentage"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Type1Name._member_map_.values()))


@JsonMap({})
class Type1(BaseModel):
    """Type1

    :param name: Simple types, defaults to None
    :type name: Type1Name, optional
    """

    def __init__(self, name: Type1Name = None):
        if name is not None:
            self.name = self._enum_matching(name, Type1Name.list(), "name")


class Type2Name(Enum):
    """An enumeration representing different categories.

    :cvar CURRENCY: "currency"
    :vartype CURRENCY: str
    """

    CURRENCY = "currency"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Type2Name._member_map_.values()))


@JsonMap({"is_fixed": "isFixed"})
class Type2(BaseModel):
    """Type2

    :param name: Currency type, defaults to None
    :type name: Type2Name, optional
    :param is_fixed: When currency is fixed, only the amount can be edited by the users, defaults to None
    :type is_fixed: bool, optional
    :param currency: Currency code, defaults to None
    :type currency: str, optional
    """

    def __init__(
        self, name: Type2Name = None, is_fixed: bool = None, currency: str = None
    ):
        if name is not None:
            self.name = self._enum_matching(name, Type2Name.list(), "name")
        if is_fixed is not None:
            self.is_fixed = is_fixed
        if currency is not None:
            self.currency = currency


class Type3Name(Enum):
    """An enumeration representing different categories.

    :cvar LIST: "list"
    :vartype LIST: str
    :cvar MULTISELECT: "multiselect"
    :vartype MULTISELECT: str
    """

    LIST = "list"
    MULTISELECT = "multiselect"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Type3Name._member_map_.values()))


@JsonMap({})
class Type3(BaseModel):
    """Type3

    :param name: List types, defaults to None
    :type name: Type3Name, optional
    :param options: List of options for the custom field, defaults to None
    :type options: List[str], optional
    """

    def __init__(self, name: Type3Name = None, options: List[str] = None):
        if name is not None:
            self.name = self._enum_matching(name, Type3Name.list(), "name")
        if options is not None:
            self.options = options


class CustomFieldsObjectTypeGuard(OneOfBaseModel):
    class_list = {"Type1": Type1, "Type2": Type2, "Type3": Type3}


CustomFieldsObjectType = Union[Type1, Type2, Type3]


class ProvidedBy(Enum):
    """An enumeration representing different categories.

    :cvar ORGANIZATION: "organization"
    :vartype ORGANIZATION: str
    :cvar WORKER: "worker"
    :vartype WORKER: str
    """

    ORGANIZATION = "organization"
    WORKER = "worker"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, ProvidedBy._member_map_.values()))


@JsonMap({"provided_by": "providedBy", "is_sensitive": "isSensitive"})
class Access(BaseModel):
    """Access settings for the custom field

    :param provided_by: Entities providing access to the custom field, defaults to None
    :type provided_by: List[ProvidedBy], optional
    :param is_sensitive: Whether the custom field is sensitive, defaults to None
    :type is_sensitive: bool, optional
    """

    def __init__(self, provided_by: List[ProvidedBy] = None, is_sensitive: bool = None):
        if provided_by is not None:
            self.provided_by = self._define_list(provided_by, ProvidedBy)
        if is_sensitive is not None:
            self.is_sensitive = is_sensitive


class WorkerTypes(Enum):
    """An enumeration representing different categories.

    :cvar EOR: "eor"
    :vartype EOR: str
    :cvar CONTRACTOR: "contractor"
    :vartype CONTRACTOR: str
    :cvar HRIS_DIRECT_EMPLOYEE: "hris_direct_employee"
    :vartype HRIS_DIRECT_EMPLOYEE: str
    :cvar DIRECT_EMPLOYEE: "direct_employee"
    :vartype DIRECT_EMPLOYEE: str
    """

    EOR = "eor"
    CONTRACTOR = "contractor"
    HRIS_DIRECT_EMPLOYEE = "hris_direct_employee"
    DIRECT_EMPLOYEE = "direct_employee"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, WorkerTypes._member_map_.values()))


@JsonMap({"worker_types": "workerTypes"})
class Settings(BaseModel):
    """Settings

    :param access: Access settings for the custom field, defaults to None
    :type access: Access, optional
    :param required: Whether the custom field is required, defaults to None
    :type required: bool, optional
    :param countries: Countries for which the custom field is applicable., defaults to None
    :type countries: List[str], optional
    :param worker_types: Worker types for which the custom field is applicable, defaults to None
    :type worker_types: List[WorkerTypes], optional
    """

    def __init__(
        self,
        access: Access = None,
        required: bool = None,
        countries: List[str] = None,
        worker_types: List[WorkerTypes] = None,
    ):
        if access is not None:
            self.access = self._define_object(access, Access)
        if required is not None:
            self.required = required
        if countries is not None:
            self.countries = countries
        if worker_types is not None:
            self.worker_types = self._define_list(worker_types, WorkerTypes)


class Placement(Enum):
    """An enumeration representing different categories.

    :cvar PROFILE: "profile"
    :vartype PROFILE: str
    :cvar CONTRACT: "contract"
    :vartype CONTRACT: str
    """

    PROFILE = "profile"
    CONTRACT = "contract"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(map(lambda x: x.value, Placement._member_map_.values()))


@JsonMap({"id_": "id", "type_": "type", "created_at": "createdAt"})
class CustomFieldsObject(BaseModel):
    """CustomFieldsObject

    :param id_: id_, defaults to None
    :type id_: str, optional
    :param name: Name of the custom field, defaults to None
    :type name: str, optional
    :param type_: type_, defaults to None
    :type type_: CustomFieldsObjectType, optional
    :param settings: settings, defaults to None
    :type settings: Settings, optional
    :param created_at: Creation date of the custom field, defaults to None
    :type created_at: str, optional
    :param placement: Valid placements for custom field, defaults to None
    :type placement: Placement, optional
    :param description: Description of the custom field, defaults to None
    :type description: str, optional
    """

    def __init__(
        self,
        id_: str = None,
        name: str = None,
        type_: CustomFieldsObjectType = None,
        settings: Settings = None,
        created_at: str = None,
        placement: Placement = None,
        description: str = None,
    ):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name
        if type_ is not None:
            self.type_ = CustomFieldsObjectTypeGuard.return_one_of(type_)
        if settings is not None:
            self.settings = self._define_object(settings, Settings)
        if created_at is not None:
            self.created_at = created_at
        if placement is not None:
            self.placement = self._enum_matching(
                placement, Placement.list(), "placement"
            )
        if description is not None:
            self.description = description



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"permission_sets": "permissionSets"})
class RoleToUpdateContainerData(BaseModel):
    """RoleToUpdateContainerData

    :param name: name, defaults to None
    :type name: str, optional
    :param description: description, defaults to None
    :type description: str, optional
    :param permission_sets: permission_sets, defaults to None
    :type permission_sets: List[str], optional
    """

    def __init__(
        self,
        name: str = None,
        description: str = None,
        permission_sets: List[str] = None,
    ):
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description
        if permission_sets is not None:
            self.permission_sets = permission_sets


@JsonMap({})
class RoleToUpdateContainer(BaseModel):
    """RoleToUpdateContainer

    :param data: data, defaults to None
    :type data: RoleToUpdateContainerData, optional
    """

    def __init__(self, data: RoleToUpdateContainerData = None):
        if data is not None:
            self.data = self._define_object(data, RoleToUpdateContainerData)



# This file was generated by liblab | https://liblab.com/

from enum import Enum
from .utils.json_map import JsonMap
from .base import BaseModel


class TimeoffReviewInternalStatus(Enum):
    """An enumeration representing different categories.

    :cvar APPROVED: "APPROVED"
    :vartype APPROVED: str
    :cvar REJECTED: "REJECTED"
    :vartype REJECTED: str
    """

    APPROVED = "APPROVED"
    REJECTED = "REJECTED"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(lambda x: x.value, TimeoffReviewInternalStatus._member_map_.values())
        )


@JsonMap({})
class TimeoffReviewInternal(BaseModel):
    """TimeoffReviewInternal

    :param status: The review decision to review the time off request with
    :type status: TimeoffReviewInternalStatus
    :param reason: The reason that the chosen status was chosen., defaults to None
    :type reason: str, optional
    """

    def __init__(self, status: TimeoffReviewInternalStatus, reason: str = None):
        self.status = self._enum_matching(
            status, TimeoffReviewInternalStatus.list(), "status"
        )
        if reason is not None:
            self.reason = reason



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({})
class UpdateWorkerWorkingLocation(BaseModel):
    """UpdateWorkerWorkingLocation

    :param working_location_id: working_location_id
    :type working_location_id: IdentifierValue
    """

    def __init__(self, working_location_id: IdentifierValue):
        self.working_location_id = IdentifierValueGuard.return_one_of(
            working_location_id
        )



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.hris_position_to_change_container import HrisPositionToChangeContainer
from ..models.get_hris_position_container import GetHrisPositionContainer


class PositionsService(BaseService):

    @cast_models
    def get_positions(self, hris_profile_id: str) -> GetHrisPositionContainer:
        """HRIS Positions from a HrisProfile.

        :param hris_profile_id: HRIS Profile ID.
        :type hris_profile_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful response.
        :rtype: GetHrisPositionContainer
        """

        Validator(str).validate(hris_profile_id)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/positions/profile/{{hrisProfile_id}}",
                self.get_default_headers(),
            )
            .add_path("hrisProfile_id", hris_profile_id)
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GetHrisPositionContainer._unmap(response)

    @cast_models
    def apply_changes_positions(self, request_body: HrisPositionToChangeContainer):
        """Apply add, edit, and delete operations to profiles.

        :param request_body: The request body.
        :type request_body: HrisPositionToChangeContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        """

        Validator(HrisPositionToChangeContainer).validate(request_body)

        serialized_request = (
            Serializer(
                f"{self.base_url}/hris/positions/apply_changes",
                self.get_default_headers(),
            )
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return response



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class Add(BaseModel):
    """Add

    :param role_id: role_id, defaults to None
    :type role_id: str, optional
    :param team_id: team_id
    :type team_id: str
    :param is_supportive: is_supportive, defaults to None
    :type is_supportive: bool, optional
    """

    def __init__(self, team_id: str, role_id: str = None, is_supportive: bool = None):
        if role_id is not None:
            self.role_id = role_id
        self.team_id = team_id
        if is_supportive is not None:
            self.is_supportive = is_supportive


@JsonMap({"id_": "id"})
class Edit(BaseModel):
    """Edit

    :param id_: id_
    :type id_: str
    :param role_id: role_id, defaults to None
    :type role_id: str, optional
    :param is_supportive: is_supportive, defaults to None
    :type is_supportive: bool, optional
    """

    def __init__(self, id_: str, role_id: str = None, is_supportive: bool = None):
        self.id_ = id_
        if role_id is not None:
            self.role_id = role_id
        if is_supportive is not None:
            self.is_supportive = is_supportive


@JsonMap({"id_": "id"})
class Delete(BaseModel):
    """Delete

    :param id_: id_
    :type id_: str
    """

    def __init__(self, id_: str):
        self.id_ = id_


@JsonMap({})
class HrisPositionToChangeContainerData(BaseModel):
    """HrisPositionToChangeContainerData

    :param add: add, defaults to None
    :type add: List[Add], optional
    :param edit: edit, defaults to None
    :type edit: List[Edit], optional
    :param delete: delete, defaults to None
    :type delete: List[Delete], optional
    :param profile_id: profile_id
    :type profile_id: str
    """

    def __init__(
        self,
        profile_id: str,
        add: List[Add] = None,
        edit: List[Edit] = None,
        delete: List[Delete] = None,
    ):
        if add is not None:
            self.add = self._define_list(add, Add)
        if edit is not None:
            self.edit = self._define_list(edit, Edit)
        if delete is not None:
            self.delete = self._define_list(delete, Delete)
        self.profile_id = profile_id


@JsonMap({})
class HrisPositionToChangeContainer(BaseModel):
    """HrisPositionToChangeContainer

    :param data: data, defaults to None
    :type data: HrisPositionToChangeContainerData, optional
    """

    def __init__(self, data: HrisPositionToChangeContainerData = None):
        if data is not None:
            self.data = self._define_object(data, HrisPositionToChangeContainerData)



# This file was generated by liblab | https://liblab.com/

from .utils.validator import Validator
from .utils.base_service import BaseService
from ..net.transport.serializer import Serializer
from ..models.utils.cast_models import cast_models
from ..models.updated_role_container import UpdatedRoleContainer
from ..models.role_to_update_container import RoleToUpdateContainer
from ..models.role_to_create_container import RoleToCreateContainer
from ..models.get_roles_container import GetRolesContainer
from ..models.created_role_container import CreatedRoleContainer


class HrisOrganizationsService(BaseService):

    @cast_models
    def get_organization_roles(self) -> GetRolesContainer:
        """Get the roles of the current organization.

        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: GetRolesContainer
        """

        serialized_request = (
            Serializer(f"{self.base_url}/roles", self.get_default_headers())
            .serialize()
            .set_method("GET")
        )

        response = self.send_request(serialized_request)

        return GetRolesContainer._unmap(response)

    @cast_models
    def create_organization_role(
        self, request_body: RoleToCreateContainer
    ) -> CreatedRoleContainer:
        """Create organization custom role.

        :param request_body: The request body.
        :type request_body: RoleToCreateContainer
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: CreatedRoleContainer
        """

        Validator(RoleToCreateContainer).validate(request_body)

        serialized_request = (
            Serializer(f"{self.base_url}/roles", self.get_default_headers())
            .serialize()
            .set_method("POST")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return CreatedRoleContainer._unmap(response)

    @cast_models
    def update_organization_role(
        self, request_body: RoleToUpdateContainer, role_id: str
    ) -> UpdatedRoleContainer:
        """Update organization custom role.

        :param request_body: The request body.
        :type request_body: RoleToUpdateContainer
        :param role_id: ID of the role.
        :type role_id: str
        ...
        :raises RequestError: Raised when a request fails, with optional HTTP status code and details.
        ...
        :return: Successful operation.
        :rtype: UpdatedRoleContainer
        """

        Validator(RoleToUpdateContainer).validate(request_body)
        Validator(str).validate(role_id)

        serialized_request = (
            Serializer(f"{self.base_url}/roles/{{role_id}}", self.get_default_headers())
            .add_path("role_id", role_id)
            .serialize()
            .set_method("PATCH")
            .set_body(request_body)
        )

        response = self.send_request(serialized_request)

        return UpdatedRoleContainer._unmap(response)



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({"id_": "id"})
class CreatedExerciseEquityContainer(BaseModel):
    """CreatedExerciseEquityContainer

    :param id_: id_
    :type id_: str
    :param status: status
    :type status: str
    :param created_at: created_at
    :type created_at: str
    :param updated_at: updated_at
    :type updated_at: str
    :param employer_approval: employer_approval
    :type employer_approval: str
    """

    def __init__(
        self,
        id_: str,
        status: str,
        created_at: str,
        updated_at: str,
        employer_approval: str,
    ):
        self.id_ = id_
        self.status = status
        self.created_at = created_at
        self.updated_at = updated_at
        self.employer_approval = employer_approval



# This file was generated by liblab | https://liblab.com/

from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class ExerciseEquityToUpdateContainer(BaseModel):
    """ExerciseEquityToUpdateContainer

    :param employer_approval: employer_approval
    :type employer_approval: str
    """

    def __init__(self, employer_approval: str):
        self.employer_approval = employer_approval



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({})
class UpdateWorkerDepartment(BaseModel):
    """UpdateWorkerDepartment

    :param department_id: department_id
    :type department_id: IdentifierValue
    """

    def __init__(self, department_id: IdentifierValue):
        self.department_id = IdentifierValueGuard.return_one_of(department_id)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel
from .employee_tax_documents_list import EmployeeTaxDocumentsList


@JsonMap({})
class EmployeeTaxDocumentsListContainer(BaseModel):
    """EmployeeTaxDocumentsListContainer

    :param data: data
    :type data: List[EmployeeTaxDocumentsList]
    """

    def __init__(self, data: List[EmployeeTaxDocumentsList]):
        self.data = self._define_list(data, EmployeeTaxDocumentsList)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .worker_bank_account_to_add import WorkerBankAccountToAdd


@JsonMap({})
class AddWorkerBankAccountContainer(BaseModel):
    """AddWorkerBankAccountContainer

    :param data: data
    :type data: WorkerBankAccountToAdd
    """

    def __init__(self, data: WorkerBankAccountToAdd):
        self.data = self._define_object(data, WorkerBankAccountToAdd)



# This file was generated by liblab | https://liblab.com/

from typing import List
from .utils.json_map import JsonMap
from .base import BaseModel


@JsonMap({})
class Teams(BaseModel):
    """Teams

    :param name: name, defaults to None
    :type name: str, optional
    :param external_id: external_id, defaults to None
    :type external_id: str, optional
    """

    def __init__(self, name: str = None, external_id: str = None):
        if name is not None:
            self.name = name
        if external_id is not None:
            self.external_id = external_id


@JsonMap({})
class OrgStructureToCreateContainerData(BaseModel):
    """OrgStructureToCreateContainerData

    :param name: name, defaults to None
    :type name: str, optional
    :param teams: teams, defaults to None
    :type teams: List[Teams], optional
    :param external_id: external_id, defaults to None
    :type external_id: str, optional
    :param enable_roles: enable_roles, defaults to None
    :type enable_roles: bool, optional
    :param is_multiple_select: is_multiple_select, defaults to None
    :type is_multiple_select: bool, optional
    """

    def __init__(
        self,
        name: str = None,
        teams: List[Teams] = None,
        external_id: str = None,
        enable_roles: bool = None,
        is_multiple_select: bool = None,
    ):
        if name is not None:
            self.name = name
        if teams is not None:
            self.teams = self._define_list(teams, Teams)
        if external_id is not None:
            self.external_id = external_id
        if enable_roles is not None:
            self.enable_roles = enable_roles
        if is_multiple_select is not None:
            self.is_multiple_select = is_multiple_select


@JsonMap({})
class OrgStructureToCreateContainer(BaseModel):
    """OrgStructureToCreateContainer

    :param data: data, defaults to None
    :type data: OrgStructureToCreateContainerData, optional
    """

    def __init__(self, data: OrgStructureToCreateContainerData = None):
        if data is not None:
            self.data = self._define_object(data, OrgStructureToCreateContainerData)



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from enum import Enum
from typing import Union
from .utils.json_map import JsonMap
from .base import BaseModel
from .base import OneOfBaseModel
from .contract_who_reports_enum import ContractWhoReportsEnum
from .meta_data_of_contract_to_create import MetaDataOfContractToCreate
from .compensation_details_of_contract_to_create_shared import (
    CompensationDetailsOfContractToCreateShared,
)
from .identifier_value import IdentifierValue, IdentifierValueGuard


@JsonMap({"id_": "id"})
class ClientLegalEntity7(BaseModel):
    """Choose the Deel legal entity for this contract. Use legal entity endpoint to retrieve a list of legal entities in your organization.

    :param id_: id_
    :type id_: IdentifierValue
    """

    def __init__(self, id_: IdentifierValue):
        self.id_ = IdentifierValueGuard.return_one_of(id_)


@JsonMap({"id_": "id"})
class ClientTeam4(BaseModel):
    """Choose the Deel team for this contract. Use teams endpoint to retrieve a list of teams in your organization.

    :param id_: id_
    :type id_: IdentifierValue
    """

    def __init__(self, id_: IdentifierValue):
        self.id_ = IdentifierValueGuard.return_one_of(id_)


@JsonMap({})
class ContractToCreatePaygMilestonesClient(BaseModel):
    """ContractToCreatePaygMilestonesClient

    :param legal_entity: Choose the Deel legal entity for this contract. Use legal entity endpoint to retrieve a list of legal entities in your organization.
    :type legal_entity: ClientLegalEntity7
    :param team: Choose the Deel team for this contract. Use teams endpoint to retrieve a list of teams in your organization.
    :type team: ClientTeam4
    """

    def __init__(self, legal_entity: ClientLegalEntity7, team: ClientTeam4):
        self.legal_entity = self._define_object(legal_entity, ClientLegalEntity7)
        self.team = self._define_object(team, ClientTeam4)


@JsonMap({"id_": "id"})
class ContractToCreatePaygMilestonesJobTitle(BaseModel):
    """Worker's job title. You can enter a custom job title or use a pre-defiled job title using the Id retrieved in job-titles endpoint.

    :param id_: Job title ID., defaults to None
    :type id_: str, optional
    :param name: Job title. Please leave it blank when entering an Id., defaults to None
    :type name: str, optional
    """

    def __init__(self, id_: str = None, name: str = None):
        if id_ is not None:
            self.id_ = id_
        if name is not None:
            self.name = name


class SeniorityId3Guard(OneOfBaseModel):
    class_list = {"float": float, "str": str}


SeniorityId3 = Union[float, str]


@JsonMap({"id_": "id"})
class ContractToCreatePaygMilestonesSeniority(BaseModel):
    """Job seniority level. Please use the seniority levels endpoint to retrieve the list of seniority levels.

    :param id_: Seniority ID., defaults to None
    :type id_: SeniorityId3, optional
    """

    def __init__(self, id_: SeniorityId3 = None):
        if id_ is not None:
            self.id_ = SeniorityId3Guard.return_one_of(id_)


@JsonMap({})
class ContractToCreatePaygMilestonesWorker(BaseModel):
    """Worker properties

    :param expected_email: Worker's email address.
    :type expected_email: str
    :param first_name: Worker's first name
    :type first_name: str
    :param last_name: Worker's last name, defaults to None
    :type last_name: str, optional
    """

    def __init__(self, expected_email: str, first_name: str, last_name: str = None):
        self.expected_email = expected_email
        self.first_name = first_name
        if last_name is not None:
            self.last_name = last_name


class ContractToCreatePaygMilestonesType(Enum):
    """An enumeration representing different categories.

    :cvar PAYG_MILESTONES: "payg_milestones"
    :vartype PAYG_MILESTONES: str
    """

    PAYG_MILESTONES = "payg_milestones"

    def list():
        """Lists all category values.

        :return: A list of all category values.
        :rtype: list
        """
        return list(
            map(
                lambda x: x.value,
                ContractToCreatePaygMilestonesType._member_map_.values(),
            )
        )


@JsonMap({"type_": "type"})
class ContractToCreatePaygMilestones(BaseModel):
    """ContractToCreatePaygMilestones

    :param title: The title of the contract.
    :type title: str
    :param country_code: Country code., defaults to None
    :type country_code: str, optional
    :param state_code: The state or province code. Use country lookup endpoint to retrieve state codes., defaults to None
    :type state_code: str, optional
    :param scope_of_work: Describe the work to be performed., defaults to None
    :type scope_of_work: str, optional
    :param special_clause: Enter any special clause you may have., defaults to None
    :type special_clause: str, optional
    :param termination_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type termination_date: str, optional
    :param client: client
    :type client: ContractToCreatePaygMilestonesClient
    :param job_title: Worker's job title. You can enter a custom job title or use a pre-defiled job title using the Id retrieved in job-titles endpoint.
    :type job_title: ContractToCreatePaygMilestonesJobTitle
    :param seniority: Job seniority level. Please use the seniority levels endpoint to retrieve the list of seniority levels., defaults to None
    :type seniority: ContractToCreatePaygMilestonesSeniority, optional
    :param notice_period: Days before to notice the termination of contract for eather party., defaults to None
    :type notice_period: float, optional
    :param who_reports: Flag to indicate who is supposed to provide regular reports, defaults to None
    :type who_reports: ContractWhoReportsEnum, optional
    :param meta: Additional custom info about a contract
    :type meta: MetaDataOfContractToCreate
    :param external_id: External Id., defaults to None
    :type external_id: str, optional
    :param worker: Worker properties, defaults to None
    :type worker: ContractToCreatePaygMilestonesWorker, optional
    :param type_: type_
    :type type_: ContractToCreatePaygMilestonesType
    :param start_date: Short date in format ISO-8601 (YYYY-MM-DD). For example 2022-12-31., defaults to None
    :type start_date: str, optional
    :param compensation_details: compensation_details
    :type compensation_details: CompensationDetailsOfContractToCreateShared
    """

    def __init__(
        self,
        title: str,
        client: ContractToCreatePaygMilestonesClient,
        job_title: ContractToCreatePaygMilestonesJobTitle,
        meta: MetaDataOfContractToCreate,
        type_: ContractToCreatePaygMilestonesType,
        compensation_details: CompensationDetailsOfContractToCreateShared,
        country_code: str = None,
        state_code: str = None,
        scope_of_work: str = None,
        special_clause: str = None,
        termination_date: str = None,
        seniority: ContractToCreatePaygMilestonesSeniority = None,
        notice_period: float = None,
        who_reports: ContractWhoReportsEnum = None,
        external_id: str = None,
        worker: ContractToCreatePaygMilestonesWorker = None,
        start_date: str = None,
    ):
        self.title = title
        if country_code is not None:
            self.country_code = self._pattern_matching(
                country_code, "^[A-Z]{2}$", "country_code"
            )
        if state_code is not None:
            self.state_code = state_code
        if scope_of_work is not None:
            self.scope_of_work = scope_of_work
        if special_clause is not None:
            self.special_clause = special_clause
        if termination_date is not None:
            self.termination_date = termination_date
        self.client = self._define_object(client, ContractToCreatePaygMilestonesClient)
        self.job_title = self._define_object(
            job_title, ContractToCreatePaygMilestonesJobTitle
        )
        if seniority is not None:
            self.seniority = self._define_object(
                seniority, ContractToCreatePaygMilestonesSeniority
            )
        if notice_period is not None:
            self.notice_period = notice_period
        if who_reports is not None:
            self.who_reports = self._enum_matching(
                who_reports, ContractWhoReportsEnum.list(), "who_reports"
            )
        self.meta = self._define_object(meta, MetaDataOfContractToCreate)
        if external_id is not None:
            self.external_id = external_id
        if worker is not None:
            self.worker = self._define_object(
                worker, ContractToCreatePaygMilestonesWorker
            )
        self.type_ = self._enum_matching(
            type_, ContractToCreatePaygMilestonesType.list(), "type_"
        )
        if start_date is not None:
            self.start_date = start_date
        self.compensation_details = self._define_object(
            compensation_details, CompensationDetailsOfContractToCreateShared
        )



# This file was generated by liblab | https://liblab.com/

from __future__ import annotations
from .utils.json_map import JsonMap
from .base import BaseModel
from .timesheet_review_to_create import TimesheetReviewToCreate


@JsonMap({})
class TimesheetReviewToCreateContainer(BaseModel):
    """TimesheetReviewToCreateContainer

    :param data: data
    :type data: TimesheetReviewToCreate
    """

    def __init__(self, data: TimesheetReviewToCreate):
        self.data = self._define_object(data, TimesheetReviewToCreate)
